using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using InfiniTec.Threading;

namespace InfiniTec.Exchange.WebDav
{
    /// <summary>
    /// This class performs a search of an Exchange public or private
    /// folder.
    /// </summary>
    /// <typeparam name="ItemType">The type of the messages returned by the searcher</typeparam>
    public class Searcher<ItemType> : SearcherBase
        where ItemType : Item, new()
    {
        /// <summary>
        /// This event is invoked when the progress of the search operation
        /// changed.
        /// </summary>
        public event EventHandler<SearchProgressChangedEventArgs<ItemType>> ProgressChanged;
        private List<ItemType> _SearchResult;

        /// <summary>
        /// Gets the search result.
        /// </summary>
        /// <value>The search result.</value>
        public IList<ItemType> SearchResult
        {
            [DebuggerStepThrough]
            get { return _SearchResult.AsReadOnly(); }
        }

        /// <summary>
        /// Synchronously starts the search operation.
        /// </summary>
        public override void Find()
        {
            _SearchResult = new List<ItemType>();
            base.Find();
        }

        /// <summary>
        /// Asynchronously starts the search operation.
        /// </summary>
        public override void FindAsync()
        {
            _SearchResult = new List<ItemType>();
            base.FindAsync();
        }

        /// <summary>
        /// Processes the partial resultset from the last search operation
        /// </summary>
        /// <param name="items">The items of the last search operation</param>
        protected override void ProcessPartialResult(IList<SimpleItem> items)
        {
            ItemType element;

            foreach (SimpleItem item in items)
            {
                element = new ItemType();
                element.BaseItem = item;

                _SearchResult.Add(element);
            }
        }

        /// <summary>
        /// Returns the search operation used to perform the search based
        /// on the specified settings.
        /// </summary>
        /// <returns></returns>
        protected override InfiniTec.Threading.AsyncOperation GetInternalSearchOperation()
        {
            return new AsyncOperation<SearchProgress<ItemType>>(SearchInternal((AsyncOperation<SearchProgress>) base.GetInternalSearchOperation()));
        }

        private IEnumerable<OperationAction> SearchInternal(AsyncOperation<SearchProgress> baseOperation)
        {
            NestedOperationStep step;
            ProgressUpdate<SearchProgress<ItemType>> status;
            List<ItemType> items;

            using (baseOperation)
            {
                step = new NestedOperationStep(baseOperation);
                status = new ProgressUpdate<SearchProgress<ItemType>>(new SearchProgress<ItemType>());

                while (baseOperation.Status == OperationStatus.Running)
                {
                    yield return step;

                    items = new List<ItemType>();
                    foreach (SimpleItem item in baseOperation.StatusInformation.Items)
                    {
                        ItemType entry;

                        entry = new ItemType();
                        entry.BaseItem = item;
                        items.Add(entry);

                        status.Status.Range = baseOperation.StatusInformation.Range;
                        status.Status.Items = items;
                        yield return status;
                    }
                }
            }
        }

        /// <summary>
        /// Allows the implementors to register events on the async operation which is about to execute.
        /// </summary>
        /// <param name="operation">The operation.</param>
        protected override void RegisterEvents(InfiniTec.Threading.AsyncOperation operation)
        {
            ((AsyncOperation<SearchProgress<ItemType>>)operation).ProgressChanged += SearchOperation_ProgressChanged;
        }

        void SearchOperation_ProgressChanged(object sender, ProgressChangedEventArgs<SearchProgress<ItemType>> e)
        {
            SearchProgressChangedEventArgs<ItemType> args;

            args = new SearchProgressChangedEventArgs<ItemType>(e.Status.Range, e.Status.Items);
            OnProgressChanged(args);

            e.Cancel = args.Cancel;

        }

        private void OnProgressChanged(SearchProgressChangedEventArgs<ItemType> e)
        {
            if (ProgressChanged != null) ProgressChanged(this, e);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Searcher"/> class.
        /// </summary>
        /// <param name="searchRoot">The root of the search operation.</param>
        /// <param name="connection">The <see cref="Connection"/> to use.</param>
        /// <param name="propertiesToLoad">The properties to load. This parameter is optional.</param>
        public Searcher(string searchRoot, Connection connection, params string[] propertiesToLoad)
            :
            base(searchRoot, connection, propertiesToLoad) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="Searcher"/> class.
        /// </summary>
        /// <param name="searchRoot">The search root.</param>
        /// <param name="connection">The connection.</param>
        /// <param name="propertiesToLoad">The properties to load.</param>
        public Searcher(Uri searchRoot, Connection connection, params string[] propertiesToLoad)
            :
            base(searchRoot, connection, propertiesToLoad) { }	        
    }
}
