using System;
using System.Collections.Generic;
using System.Text;

namespace InfiniTec.Exchange.WebDav
{
    /// <summary>
    /// This class defines various properties used for task items.
    /// </summary>
    [SchemaClass(typeof(NamedProperties.Task))]
    public class TaskItem : Item
    {
        private static string[] _RequiredFields;

        /// <summary>
        /// Initializes the <see cref="TaskItem"/> class.
        /// </summary>
        static TaskItem()
        {
            _RequiredFields = SchemaHelper.ExtractFieldValues(typeof(NamedProperties.Task));
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TaskItem"/> class.
        /// </summary>
        public TaskItem(): base()
        {
            Initialize();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TaskItem"/> class.
        /// </summary>
        /// <param name="path">The path of the task.</param>
        /// <param name="connection">The connection.</param>
        public TaskItem(string path, Connection connection)
            : base(path, connection)
        {
            Initialize();
        }

        private void Initialize()
        {
            AddRequiredFields(_RequiredFields);
        }

        /// <summary>
        /// This method is called before the save operation is called.
        /// </summary>
        protected override void PreSave()
        {
            base.PreRefresh();

            if (!PercentComplete.HasValue) PercentComplete = 0;
            if (!IconIndex.HasValue) IconIndex = global::InfiniTec.Exchange.IconIndex.TaskItem;
            if (!Status.HasValue) Status = TaskStatus.NotStarted;
        }

        /// <summary>
        /// Gets the default content class for this class.
        /// </summary>
        /// <param name="outlookMessageClass">The outlook message class (e.g. IPM.NOTE).</param>
        /// <param name="contentClass">The WebDAV content class (e.g. urn:content-classes:message).</param>
        protected override void GetDefaultContentClass(out string outlookMessageClass, out string contentClass)
        {
            outlookMessageClass = "IPM.Task";
            contentClass = "urn:content-classes:task";
        }

        /// <summary>
        /// Gets or sets the actual work.
        /// </summary>
        /// <value>The actual work.</value>
        public int? ActualWork
        {
            get { return GetNullablePropertyValue<int>(NamedProperties.Task.ActualWork); }
            set { SetNullablePropertyValue<int>(NamedProperties.Task.ActualWork, value); }
        }
        /// <summary>
        /// Gets or sets the assigned time.
        /// </summary>
        /// <value>The assigned time.</value>
        public DateTime? AssignedTime
        {
            get { return GetNullablePropertyValue<DateTime>(NamedProperties.Task.AssignedTime); }
            set { SetNullablePropertyValue<DateTime>(NamedProperties.Task.AssignedTime, value); }
        }

        /// <summary>
        /// Gets or sets the billing information.
        /// </summary>
        /// <value>The billing information.</value>
        public string BillingInformation
        {
            get { return GetPropertyValue<string>(NamedProperties.Task.BillingInformation); }
            set { SetPropertyValue<string>(NamedProperties.Task.BillingInformation, value); }
        }

        /// <summary>
        /// Gets or sets the change count.
        /// </summary>
        /// <value>The change count.</value>
        public int? ChangeCount
        {
            get { return GetNullablePropertyValue<int>(NamedProperties.Task.ChangeCount); }
            set { SetNullablePropertyValue<int>(NamedProperties.Task.ChangeCount, value); }
        }

        /// <summary>
        /// Gets a list of the companies.
        /// </summary>
        /// <value>A List containing the companies.</value>
        public IList<string> Companies
        {
            get
            {
                EnsureMultiValuedProperty<string>(NamedProperties.Task.Companies);

                return Properties.GetMultivaluedProperty<string>(NamedProperties.Task.Companies).Values;
            }
        }

        /// <summary>
        /// Gets or sets the complete date.
        /// </summary>
        /// <value>The complete date.</value>
        public DateTime? CompleteDate
        {
            get { return GetNullablePropertyValue<DateTime>(NamedProperties.Task.CompleteDate); }
            set { SetNullablePropertyValue<DateTime>(NamedProperties.Task.CompleteDate, value); }
        }

        /// <summary>
        /// Gets the contacts.
        /// </summary>
        /// <value>The contacts.</value>
        public IList<string> Contacts
        {
            get
            {
                EnsureMultiValuedProperty<string>(NamedProperties.Task.Contacts);

                return Properties.GetMultivaluedProperty<string>(NamedProperties.Task.Contacts).Values;
            }
        }

        /// <summary>
        /// Gets or sets the state of the delegation.
        /// </summary>
        /// <value>The state of the delegation.</value>
        public TaskDelegationState? DelegationState { 
            get { return (TaskDelegationState) GetNullablePropertyValue<int>(NamedProperties.Task.DelegationState); }
            set { SetNullablePropertyValue<int>(NamedProperties.Task.DelegationState, (int) value); }
        }

        /// <summary>
        /// Gets or sets the delegator.
        /// </summary>
        /// <value>The delegator.</value>
        public string Delegator
        {
            get { return GetPropertyValue<string>(NamedProperties.Task.Delegator); }
            set { SetPropertyValue<string>(NamedProperties.Task.Delegator, value); }
        }

        /// <summary>
        /// Gets or sets the due date.
        /// </summary>
        /// <value>The due date.</value>
        public DateTime? DueDate
        {
            get
            {
                if (Properties.Contains(NamedProperties.Task.UtcDueDate)) return GetNullablePropertyValue<DateTime>(NamedProperties.Task.UtcDueDate);
                else if (Properties.Contains(NamedProperties.Task.DueDate)) return GetNullablePropertyValue<DateTime>(NamedProperties.Task.DueDate);
                else return null;
            }
            set
            {
                SetNullablePropertyValue<DateTime>(NamedProperties.Task.DueDate, value);
                SetNullablePropertyValue<DateTime>(NamedProperties.Task.UtcDueDate, value);
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the assignment is editable.
        /// </summary>
        /// <value>
        /// 	<see langword="true"/> if the assignment of this instance is editable; otherwise, <see langword="false"/>.
        /// </value>
        public int? IsAssignmentEditable
        {
            get { return GetNullablePropertyValue<int>(NamedProperties.Task.IsAssignmentEditable); }
            set { SetNullablePropertyValue<int>(NamedProperties.Task.IsAssignmentEditable, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is complete.
        /// </summary>
        /// <value>
        /// 	<see langword="true"/> if this instance is complete; otherwise, <see langword="false"/>.
        /// </value>
        public bool? IsComplete
        {
            get { return GetNullablePropertyValue<bool>(NamedProperties.Task.IsComplete); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is recurring.
        /// </summary>
        /// <value>
        /// 	<see langword="true"/> if this instance is recurring; otherwise, <see langword="false"/>.
        /// </value>
        public bool? IsRecurring
        {
            get { return GetNullablePropertyValue<bool>(NamedProperties.Task.IsRecurring); }
            set { SetNullablePropertyValue<bool>(NamedProperties.Task.IsRecurring, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is a team task.
        /// </summary>
        /// <value>
        /// 	<see langword="true"/> if this instance is a team task; otherwise, <see langword="false"/>.
        /// </value>
        public bool? IsTeamTask
        {
            get { return GetNullablePropertyValue<bool>(NamedProperties.Task.IsTeamTask); }
            set { SetNullablePropertyValue<bool>(NamedProperties.Task.IsTeamTask, value); }
        }

        /// <summary>
        /// Gets or sets the mileage.
        /// </summary>
        /// <value>The mileage.</value>
        public string Mileage
        {
            get { return GetPropertyValue<string>(NamedProperties.Task.Mileage); }
            set { SetPropertyValue<string>(NamedProperties.Task.Mileage, value); }
        }

        /// <summary>
        /// Gets or sets the owner.
        /// </summary>
        /// <value>The owner.</value>
        public string Owner
        {
            get { return GetPropertyValue<string>(NamedProperties.Task.Owner); }
            set { SetPropertyValue<string>(NamedProperties.Task.Owner, value); }
        }

        /// <summary>
        /// Gets or sets the percent complete.
        /// </summary>
        /// <value>The percent complete.</value>
        public float? PercentComplete
        {
            get { return GetNullablePropertyValue<float>(NamedProperties.Task.PercentComplete); }
            set { SetNullablePropertyValue<float>(NamedProperties.Task.PercentComplete, value); }
        }

        /// <summary>
        /// Gets or sets the start date.
        /// </summary>
        /// <value>The start date.</value>
        public DateTime? StartDate
        {
            get { return GetNullablePropertyValue<DateTime>(NamedProperties.Task.StartDate); }
            set { SetNullablePropertyValue<DateTime>(NamedProperties.Task.StartDate, value); }
        }

        /// <summary>
        /// Gets or sets the status.
        /// </summary>
        /// <value>The status.</value>
        public TaskStatus? Status
        {
            get { return (TaskStatus?) GetNullablePropertyValue<int>(NamedProperties.Task.TaskStatus); }
            set { SetNullablePropertyValue<int>(NamedProperties.Task.TaskStatus, (int?) value); }
        }

        /// <summary>
        /// Gets or sets the status description.
        /// </summary>
        /// <value>The status description.</value>
        public string StatusDescription
        {
            get { return GetPropertyValue<string>(NamedProperties.Task.StatusDescription); }
            set { SetPropertyValue<string>(NamedProperties.Task.StatusDescription, value); }
        }

        /// <summary>
        /// Gets or sets the total work.
        /// </summary>
        /// <value>The total work.</value>
        public int? TotalWork
        {
            get { return GetNullablePropertyValue<int>(NamedProperties.Task.TotalWork); }
            set { SetNullablePropertyValue<int>(NamedProperties.Task.TotalWork, value); }
        }

        /// <summary>
        /// Marks the task as complete and sets the completion date accordingly.
        /// </summary>
        /// <param name="dateCompleted">The date/time when the task has been completed</param>
        public void SetComplete(DateTime dateCompleted)
        {
            Status = TaskStatus.NotStarted;
            SetNullablePropertyValue<DateTime>(NamedProperties.Task.FlagCompleteTime, dateCompleted);
            PercentComplete = 1;
            CompleteDate = dateCompleted.Date;
            IconIndex = global::InfiniTec.Exchange.IconIndex.TaskItem;
        }
    }
}
